/*
 * Copyright (C) 1997-2008 Andre Noll <maan@systemlinux.org>
 *
 * Licensed under the GPL v2. For licencing details see COPYING.
 */

/** \file client.h common client functions and exported symbols from client_common.c */

#include <openssl/rc4.h>

/**
 * the different states of a connection from the view of the client
 */
enum {
	/** tcp connection is established */
	CL_CONNECTED,
	/** server sends the welcome message */
	CL_RECEIVED_WELCOME,
	/** client sends the authentification request */
	CL_SENT_AUTH,
	/** server sends a challenge */
	CL_RECEIVED_CHALLENGE,
	/** clientd solves the challenge and sends the result */
	CL_SENT_CH_RESPONSE,
	/** server accepts this authentication */
	CL_RECEIVED_PROCEED,
	/** client sends the command */
	CL_SENT_COMMAND,
	/** server expects data */
	CL_SENDING,
	/** client expects data */
	CL_RECEIVING,
};

/** size of the receiving buffer */
#define CLIENT_BUFSIZE 8192

/**
 * data specific to a client task
 */
struct client_task {
	/** the state of the connection */
	int status;
	/** the file descriptor */
	int fd;
	/** the configuration (including the command) */
	struct client_args_info conf;
	/** the config file for client options */
	char *config_file;
	/** the RSA private key */
	char *key_file;
	/** paraslash user name */
	char *user;
	/** session key for receiving data */
	RC4_KEY rc4_recv_key;
	/** session key for sending data */
	RC4_KEY rc4_send_key;
	/** the client task structure */
	struct task task;
	/** the buffer used for handshake and receiving */
	char buf[CLIENT_BUFSIZE];
	/** number of bytes loaded in \p buf */
	size_t loaded;
	/** non-zero if the pre_select hook added \p fd to the read fd set */
	int check_r;
	/** non-zero if the pre_select hook added \p fd to the write fd set */
	int check_w;
	/** the decrypted challenge */
	long unsigned challenge_nr;
	/** pointer to the data to be sent to para_server */
	char *inbuf;
	/** number of bytes loaded in \p inbuf */
	size_t *in_loaded;
	/** Non-zero if input task encountered an eof or an error condition. */
	int *in_error;
};

void client_close(struct client_task *ct);
int client_open(int argc, char *argv[], struct client_task **ct);
