/*
 * Copyright (C) 2006-2008 Andre Noll <maan@systemlinux.org>
 *
 * Licensed under the GPL v2. For licencing details see COPYING.
 */

/** \file error.h List of error messages for all subsystems. */

/** \cond */

/* List of all subsystems that use paraslash's error facility. */
DEFINE_ERRLIST_OBJECT_ENUM;

/* these do not need error handling (yet) */
#define SERVER_ERRORS
#define WAV_ERRORS
#define COMPRESS_ERRORS
#define TIME_ERRORS
#define CLOSE_ON_FORK_ERRORS
#define DAEMON_ERRORS
#define ORTP_SEND_ERRORS
#define GUI_ERRORS
#define RINGBUFFER_ERRORS
#define SCORE_ERRORS
#define SHA1_ERRORS
#define RBTREE_ERRORS
#define RECV_ERRORS
#define SEND_COMMON_ERRORS
#define STDOUT_ERRORS


extern const char **para_errlist[];

#define FADE_ERRORS \
	PARA_ERROR(FADE_SYNTAX, "fade syntax error"), \


#define CLIENT_ERRORS \
	PARA_ERROR(TASK_STARTED, "task started"), \


#define AFH_ERRORS \
	PARA_ERROR(AFH_SYNTAX, "afh syntax error"), \
	PARA_ERROR(AFH_SHORT_WRITE, "afh short write"), \


#define AFH_COMMON_ERRORS \
	PARA_ERROR(AUDIO_FORMAT, "audio format not recognized"), \


#define ACL_ERRORS \
	PARA_ERROR(ACL_PERM, "access denied by acl"), \


#define FSCK_ERRORS \
	PARA_ERROR(FSCK_SYNTAX, "fsck syntax error"), \
	PARA_ERROR(RANGE_VIOLATION, "range violation detected, very bad"), \
	PARA_ERROR(NOT_A_REGULAR_FILE, "not a regular file"), \


#define OSL_ERRORS \
	PARA_ERROR(BAD_DB_DIR, "invalid database directory"), \
	PARA_ERROR(NO_COLUMN_DESC, "missing column description"), \
	PARA_ERROR(BAD_NAME, "invalid name for a column/table"), \
	PARA_ERROR(BAD_STORAGE_TYPE, "invalid storage type"), \
	PARA_ERROR(BAD_STORAGE_FLAGS, "invalid storage flags"), \
	PARA_ERROR(NO_COLUMN_NAME, "missing column name"), \
	PARA_ERROR(NO_COLUMNS, "at least one column required"), \
	PARA_ERROR(BAD_COLUMN_NAME, "invalid name for a table column"), \
	PARA_ERROR(NO_UNIQUE_RBTREE_COLUMN, "need at least one mapped column with OSL_UNIQE and OSL_RBTREE OSL"), \
	PARA_ERROR(NO_RBTREE_COL, "at least one column needs an rbtree"), \
	PARA_ERROR(DUPLICATE_COL_NAME, "column name given twice"), \
	PARA_ERROR(BAD_STORAGE_SIZE, "invalid storage size"), \
	PARA_ERROR(NO_COMPARE_FUNC, "missing compare function"), \
	PARA_ERROR(BAD_DATA_SIZE, "wrong data size for fixed-size column"), \
	PARA_ERROR(NOT_MAPPED, "file not mapped"), \
	PARA_ERROR(ALREADY_MAPPED, "file already mapped"), \
	PARA_ERROR(BAD_SIZE, "invalid size specified"), \
	PARA_ERROR(TRUNC, "failed to truncate file"), \
	PARA_ERROR(BAD_TABLE, "table not open"), \
	PARA_ERROR(BAD_TABLE_DESC, "invalid table description"), \
	PARA_ERROR(RB_KEY_EXISTS, "key already exists in rbtree"), \
	PARA_ERROR(RB_KEY_NOT_FOUND, "key not found in rbtree"), \
	PARA_ERROR(BAD_ROW_NUM, "invalid row number"), \
	PARA_ERROR(INDEX_CORRUPTION, "index corruption detected"), \
	PARA_ERROR(INVALID_OBJECT, "reference to invalid object"), \
	PARA_ERROR(STAT, "can not stat file"), \
	PARA_ERROR(WRITE, "write error"), \
	PARA_ERROR(LSEEK, "lseek error"), \
	PARA_ERROR(BUSY, "table is busy"), \
	PARA_ERROR(SHORT_TABLE, "table too short"), \
	PARA_ERROR(NO_MAGIC, "missing table header magic"), \
	PARA_ERROR(VERSION_MISMATCH, "table version not supported"), \
	PARA_ERROR(BAD_COLUMN_NUM, "invalid column number"), \
	PARA_ERROR(BAD_TABLE_FLAGS, "invalid flags in table description"), \
	PARA_ERROR(BAD_ROW, "invalid row"), \



#define AFS_ERRORS \
	PARA_ERROR(BAD_TABLE_NAME, "invalid table"), \
	PARA_ERROR(INPUT_TOO_LARGE, "input too large for stdin command"), \
	PARA_ERROR(AFS_SYNTAX, "afs syntax error"), \
	PARA_ERROR(AFS_SIGNAL, "afs caught deadly signal"), \
	PARA_ERROR(AFS_SOCKET, "afs socket not writable"), \
	PARA_ERROR(AFS_SHORT_READ, "short read from afs socket"), \


#define MOOD_ERRORS \
	PARA_ERROR(MOOD_SYNTAX, "mood syntax error"), \
	PARA_ERROR(NO_MOOD, "no mood available"), \
	PARA_ERROR(NOT_ADMISSIBLE, "file is not admissible"), \


#define ATTRIBUTE_ERRORS \
	PARA_ERROR(ATTR_SYNTAX, "attribute syntax error"), \
	PARA_ERROR(NO_ATTRIBUTES, "no attributes defined yet"), \
	PARA_ERROR(ATT_TABLE_FULL, "no more space left in attribute table"), \


#define BLOB_ERRORS \
	PARA_ERROR(BLOB_SYNTAX, "blob syntax error"), \
	PARA_ERROR(DUMMY_ROW, "attempted to access blob dummy object"), \


#define PLAYLIST_ERRORS \
	PARA_ERROR(PLAYLIST_LOADED, ""), /* not really an error */ \
	PARA_ERROR(PATH_FOUND, ""), /* not really an error */ \
	PARA_ERROR(PLAYLIST_EMPTY, "attempted to load empty playlist"), \


#define AFT_ERRORS \
	PARA_ERROR(BAD_AFSI, "invaiid afs info"), \
	PARA_ERROR(LOCALTIME, "localtime() failed"), \
	PARA_ERROR(STRFTIME, "strftime() failed"), \
	PARA_ERROR(BAD_PATH, "invalid path"), \
	PARA_ERROR(BAD_SORT, "invalid sorting method"), \
	PARA_ERROR(FNMATCH, "fnmatch error"), \
	PARA_ERROR(NO_MATCH, "no matches"), \
	PARA_ERROR(NO_AFHI, "audio format handler info required"), \
	PARA_ERROR(AFT_SYNTAX, "audio file table syntax error"), \
	PARA_ERROR(HASH_MISMATCH, "hash mismatch, consider re-add"), \


#define USER_LIST_ERRORS \
	PARA_ERROR(USERLIST, "failed to open user list file"), \


#define OSX_WRITE_ERRORS \
	PARA_ERROR(STREAM_FORMAT, "could not set stream format"), \
	PARA_ERROR(ADD_CALLBACK, "can not add callback"), \
	PARA_ERROR(OPEN_COMP, "OpenAComponent() error"), \
	PARA_ERROR(UNIT_INIT, "AudioUnitInitialize() error"), \
	PARA_ERROR(UNIT_START, "AudioUnitStart() error"), \
	PARA_ERROR(DEFAULT_COMP, "can not find default audio output component"), \


#define AUDIOC_ERRORS \
	PARA_ERROR(AUDIOC_SYNTAX, "audioc syntax error"), \
	PARA_ERROR(AUDIOC_READ, "audioc read error"), \
	PARA_ERROR(AUDIOC_WRITE, "audioc write error"), \
	PARA_ERROR(AUDIOC_OVERRUN, "audioc buffer overrun"), \


#define CLIENT_COMMON_ERRORS \
	PARA_ERROR(CLIENT_SYNTAX, "syntax error"), \
	PARA_ERROR(INVALID_CHALLENGE, "did not receive valid challenge"), \
	PARA_ERROR(NO_CONFIG, "config file not found"), \
	PARA_ERROR(CLIENT_AUTH, "authentication failed"), \
	PARA_ERROR(SERVER_EOF, "connection closed by para_server"), \


#define SCHED_ERRORS \
	PARA_ERROR(TASK_KILLED, "task killed"), \
	PARA_ERROR(TASK_UNREGISTERED, "task has been unscheduled"), \
	PARA_ERROR(NO_SUCH_TASK, "task not found"), \
	PARA_ERROR(NOT_INITIALIZED, "scheduler not yet initialized"), \


#define STDIN_ERRORS \
	PARA_ERROR(STDIN_EOF, "end of file"), \



#define NET_ERRORS \
	PARA_ERROR(NAME_TOO_LONG, "name too long for struct sockaddr_un"), \
	PARA_ERROR(ADDRESS_LOOKUP, "address lookup / socket creation failed"), \
	PARA_ERROR(CHMOD, "failed to set socket mode"), \
	PARA_ERROR(SENDMSG, "sendmsg() failed"), \
	PARA_ERROR(RECVMSG, "recvmsg() failed"), \
	PARA_ERROR(SCM_CREDENTIALS, "did not receive SCM credentials"), \
	PARA_ERROR(RECV_PATTERN, "did not receive expected pattern"), \


#define ORTP_RECV_ERRORS \
	PARA_ERROR(MSG_TO_BUF, "failed to extract rtp packet"), \
	PARA_ERROR(ORTP_SYNTAX, "ortp syntax error"), \
	PARA_ERROR(TOO_MANY_BAD_CHUNKS, "too many consecutive bad chunks"), \
	PARA_ERROR(INVALID_HEADER, "invalid header packet"), \
	PARA_ERROR(OVERRUN, "output buffer overrun"), \
	PARA_ERROR(ORTP_RECV_EOF, "ortp_recv: end of file"), \


#define HTTP_RECV_ERRORS \
	PARA_ERROR(HTTP_RECV_OVERRUN, "http_recv: output buffer overrun"), \


#define RECV_COMMON_ERRORS \
	PARA_ERROR(RECV_SYNTAX, "recv syntax error"), \
	PARA_ERROR(RECV_EOF, "end of file"), \


#define AUDIOD_ERRORS \
	PARA_ERROR(NO_MORE_SLOTS, "no more empty slots"), \
	PARA_ERROR(MISSING_COLON, "syntax error: missing colon"), \
	PARA_ERROR(UNSUPPORTED_AUDIO_FORMAT, "given audio format not supported"), \
	PARA_ERROR(NOT_PLAYING, "not playing"), \
	PARA_ERROR(AUDIOD_OFF, "audiod switched off"), \
	PARA_ERROR(STATUS_TIMEOUT, "timeout reading server status"), \


#define AUDIOD_COMMAND_ERRORS \
	PARA_ERROR(CLIENT_WRITE, "client write error"), \
	PARA_ERROR(AUDIOD_SYNTAX, "syntax error"), \
	PARA_ERROR(UCRED_PERM, "permission denied"), \
	PARA_ERROR(INVALID_AUDIOD_CMD, "invalid command"), \


#define FILTER_CHAIN_ERRORS \
	PARA_ERROR(UNSUPPORTED_FILTER, "given filter not supported"), \
	PARA_ERROR(BAD_FILTER_OPTIONS, "invalid filter option given"), \
	PARA_ERROR(FC_EOF, "filter chain: eof"), \


#define STAT_ERRORS \
	PARA_ERROR(TOO_MANY_CLIENTS, "maximal number of stat clients exceeded"), \
	PARA_ERROR(UNKNOWN_STAT_ITEM, "status item not recognized"), \


#define OGGDEC_ERRORS \
	PARA_ERROR(OGGDEC_READ, "read from media returned an error"), \
	PARA_ERROR(OGGDEC_NOTVORBIS, "bitstream is not vorbis data"), \
	PARA_ERROR(OGGDEC_VERSION, "vorbis version mismatch"), \
	PARA_ERROR(OGGDEC_BADHEADER, "invalid vorbis bitstream header"), \
	PARA_ERROR(OGGDEC_FAULT, "bug or heap/stack corruption"), \
	PARA_ERROR(OGGDEC_BADLINK, "invalid stream section or requested link corrupt"), \


#define GRAB_CLIENT_ERRORS \
	PARA_ERROR(PEDANTIC_GRAB, "fd not ready and pedantic grab requested"), \
	PARA_ERROR(GC_WRITE, "grab client write error"), \
	PARA_ERROR(BAD_GC_SLOT, "invalid slot requested by grab client"), \
	PARA_ERROR(BAD_GC_FILTER_NUM, "invalid filter number given"), \
	PARA_ERROR(GC_SYNTAX, "grab client syntax error"), \
	PARA_ERROR(GC_HELP_GIVEN, ""), /* not really an error */ \
	PARA_ERROR(GC_VERSION_GIVEN, ""), /* not really an error */ \


#define MP3DEC_ERRORS \
	PARA_ERROR(MAD_FRAME_DECODE, "mad frame decode error"), \
	PARA_ERROR(MP3DEC_OVERRUN, "mp3 output buffer overrun"), \


#define FILTER_ERRORS \
	PARA_ERROR(NO_FILTERS, "at least one filter must be given"), \
	PARA_ERROR(FILTER_SYNTAX, "syntax error"), \


#define SIGNAL_ERRORS \
	PARA_ERROR(SIGNAL_SIG_ERR, "signal() returned SIG_ERR"), \


#define STRING_ERRORS \
	PARA_ERROR(ATOI_OVERFLOW, "value too large"), \
	PARA_ERROR(STRTOLL, "unknown strtoll error"), \
	PARA_ERROR(ATOI_NO_DIGITS, "no digits found in string"), \
	PARA_ERROR(ATOI_JUNK_AT_END, "further characters after number"), \


#define EXEC_ERRORS \
	PARA_ERROR(DUP_PIPE, "exec error: can not create pipe"), \


#define MP3_AFH_ERRORS \
	PARA_ERROR(FRAME, "invalid mp3 frame"), \
	PARA_ERROR(FRAME_LENGTH, "invalid frame length"), \
	PARA_ERROR(MP3_INFO, "could not read mp3 info"), \
	PARA_ERROR(HEADER_FREQ, "invalid header frequency"), \
	PARA_ERROR(HEADER_BITRATE, "invalid header bitrate"), \


#define AAC_AFH_ERRORS \
	PARA_ERROR(STSZ, "did not find stcz atom"), \
	PARA_ERROR(MP4ASC, "audio spec config error"), \
	PARA_ERROR(AAC_AFH_INIT, "failed to init aac decoder"), \


#define AAC_COMMON_ERRORS \
	PARA_ERROR(ESDS, "did not find esds atom"), \
	PARA_ERROR(STCO, "did not find stco atom"), \


#define OGG_AFH_ERRORS \
	PARA_ERROR(OGG_READ, "ogg read error"), \
	PARA_ERROR(SYNC_PAGEOUT, "ogg sync page-out error (no ogg file?)"), \
	PARA_ERROR(STREAM_PAGEIN, "ogg stream page-in error (first page)"), \
	PARA_ERROR(STREAM_PACKETOUT, "ogg stream packet-out error (first packet)"), \
	PARA_ERROR(VORBIS, "vorbis synthesis header-in error (not vorbis?)"), \
	PARA_ERROR(OGG_INFO, "ov_info error"), \
	PARA_ERROR(OGG_VERSION, "unsupported ogg version"), \
	PARA_ERROR(OGG_BAD_HEADER, "invalid ogg vorbis header"), \
	PARA_ERROR(OGG_UNKNOWN_ERROR, "unknown ogg vorbis error"), \


#define VSS_ERRORS \
	PARA_ERROR(CHUNK, "unable to get chunk"), \
	PARA_ERROR(NOFD, "did not receive open fd from afs"), \


#define CRYPT_ERRORS \
	PARA_ERROR(PRIVATE_KEY, "can not read private key"), \
	PARA_ERROR(PUBLIC_KEY, "can not read public key"), \
	PARA_ERROR(RSA, "RSA error"), \
	PARA_ERROR(ENCRYPT, "encrypt error"), \
	PARA_ERROR(DECRYPT, "decrypt error"), \
	PARA_ERROR(CHALLENGE, "failed to read challenge"), \


#define HTTP_SEND_ERRORS \
	PARA_ERROR(MAX_CLIENTS, "maximal number of clients exceeded"), \


#define COMMAND_ERRORS \
	PARA_ERROR(COMMAND_SYNTAX, "syntax error in command"), \
	PARA_ERROR(AUTH, "did not receive auth request"), \
	PARA_ERROR(NO_AUDIO_FILE, "no audio file"), \
	PARA_ERROR(BAD_CMD, "invalid command"), \
	PARA_ERROR(PERM, "permission denied"), \
	PARA_ERROR(LOCK, "lock error"), \
	PARA_ERROR(SENDER_CMD, "command not supported by this sender"), \
	PARA_ERROR(SERVER_CRASH, "para_server crashed -- can not live without it"), \
	PARA_ERROR(BAD_USER, "you don't exist. Go away."), \


#define IPC_ERRORS \
	PARA_ERROR(SEM_GET, "failed to create semaphore"), \
	PARA_ERROR(SEM_REMOVE, "can not remove semaphore"), \
	PARA_ERROR(SHM_GET, "failed to allocate shared memory area"), \
	PARA_ERROR(SHM_DESTROY, "failed to destroy shared memory area"), \
	PARA_ERROR(SHM_ATTACH, "can not attach shared memory area"), \
	PARA_ERROR(SHM_DETACH, "can not detach shared memory area"), \


#define DCCP_RECV_ERRORS \
	PARA_ERROR(DCCP_OVERRUN, "dccp output buffer buffer overrun"), \


#define DCCP_SEND_ERRORS \


#define FD_ERRORS \
	PARA_ERROR(FGETS, "fgets error"), \
	PARA_ERROR(EMPTY, "file empty"), \
	PARA_ERROR(MMAP, "mmap error"), \


#define WRITE_ERRORS \
	PARA_ERROR(WRITE_SYNTAX, "para_write syntax error"), \
	PARA_ERROR(NO_WAV_HEADER, "wave header not found"), \
	PARA_ERROR(WAV_HEADER_SUCCESS, "successfully read wave header"), \
	PARA_ERROR(NO_DELAY, "no initial delay"), \
	PARA_ERROR(DELAY_TIMEOUT, "initial delay timeout"), \


#define ALSA_WRITE_ERRORS \
	PARA_ERROR(BROKEN_CONF, "Broken alsa configuration"), \
	PARA_ERROR(ACCESS_TYPE, "alsa access type not available"), \
	PARA_ERROR(SAMPLE_FORMAT, "sample format not available"), \
	PARA_ERROR(CHANNEL_COUNT, "channels count not available"), \
	PARA_ERROR(HW_PARAMS, "unable to install hw params"), \
	PARA_ERROR(SW_PARAMS, "unable to install sw params"), \
	PARA_ERROR(BAD_PERIOD, "can not use period equal to buffer size"), \
	PARA_ERROR(ALSA_WRITE, "alsa write error"), \
	PARA_ERROR(PCM_OPEN, "unable to open pcm"), \
	PARA_ERROR(SND_PCM_INFO, "pcm info error"), \
	PARA_ERROR(GET_BUFFER_TIME, "snd_pcm_hw_params_get_buffer_time_max() failed"), \
	PARA_ERROR(SET_BUFFER_TIME, "snd_pcm_hw_params_set_buffer_time_near() failed"), \
	PARA_ERROR(SET_RATE, "snd_pcm_hw_params_set_rate_near failed"), \
	PARA_ERROR(START_THRESHOLD, "snd_pcm_sw_params_set_start_threshold() failed"), \
	PARA_ERROR(STOP_THRESHOLD, "snd_pcm_sw_params_set_stop_threshold() failed"), \


#define FILE_WRITE_ERRORS \
	PARA_ERROR(FW_WRITE, "file writer write error"), \
	PARA_ERROR(FW_OPEN, "file writer: can not open output file"), \
	PARA_ERROR(FW_NO_FILE, "task started without open file"), \


#define WRITE_COMMON_ERRORS \
	PARA_ERROR(WRITE_COMMON_SYNTAX, "syntax error in write option"), \


#define AACDEC_ERRORS \
	PARA_ERROR(AACDEC_INIT, "failed to init aac decoder"), \
	PARA_ERROR(AAC_DECODE, "aac decode error"), \
	PARA_ERROR(AAC_OVERRUN, "aac output buffer overrun"), \


#define CHUNK_QUEUE_ERRORS \
	PARA_ERROR(QUEUE, "packet queue overrun"), \


/** \endcond */

/**
 * The subsystem shift.
 *
 * 255 error codes ought to be enough for every subsystem. Use the higher bits
 * of the return value to encode the subsystem number.
 */
#define SS_SHIFT 8

/**
 * Compute the subsystem offset.
 *
 * It is given by x * 2**8 where \a x is the subsystem number.
 */
#define SS_OFFSET(ss) (SS_ ## ss << SS_SHIFT)

/**
 * Make the enum of all errors of one subsystem.
 *
 * As zero should not be an error, we define a dummy enum entry with value
 * 2**ss. That lets the real errors start at 2**ss + 1.
 */
#define SS_ENUM(ss) enum {\
	E_ ## ss ## _DUMMY = SS_OFFSET(ss), \
	ss ## _ERRORS}

/**
 * Determine the subsystem number from the error number.
 *
 * Easy, it's just \a num / 2**8.
 */
#define ERRNUM_TO_SS(num) ((num) >> SS_SHIFT)

/**
 * Determine the index of an error number.
 *
 * Also easy: It's the lower 8 bits of num.
 */
#define ERRNUM_TO_INDEX(num) (((1 << SS_SHIFT) - 1) & (num))

/**
 * This bit indicates whether a number is considered a system error number
 * If yes, the system errno is just the result of clearing this bit from
 * the given number.
 */
#define SYSTEM_ERROR_BIT 30

/** Check whether the system error bit is set. */
#define IS_SYSTEM_ERROR(num) (!!((num) & (1 << SYSTEM_ERROR_BIT)))

/** Set the system error bit for the given number. */
#define ERRNO_TO_PARA_ERROR(num) ((num) | (1 << SYSTEM_ERROR_BIT))

/** Check whether a given number is a system error number.
 *
 * \param num The value to be checked.
 * \param _errno The system error number.
 *
 * \return True if \a num is paraslash's representation of the system
 * error identified by \a _errno.
 */
_static_inline_ int is_errno(int num, int _errno)
{
	assert(num > 0 && _errno > 0);
	return ERRNO_TO_PARA_ERROR(_errno) == num;
}

/**
 * Paraslash's version of strerror(3).
 *
 * \param num The error number.
 *
 * \return The error text of \a num.
 */
_static_inline_ const char *para_strerror(int num)
{
	assert(num > 0);
	if (IS_SYSTEM_ERROR(num))
		return strerror((num) & ((1 << SYSTEM_ERROR_BIT) - 1));
	else
		return para_errlist[ERRNUM_TO_SS(num)][ERRNUM_TO_INDEX(num)];
}
/**
 * Define the error list for one subsystem.
 #
 * The first entry (index 0) is always the name of the subsystem
 * Used by macros in config.h (generated by configure).
 */
#define DEFINE_ERRLIST(ss) const char * ss ## _ERRLIST[] = {#ss, ss ## _ERRORS}

/**
 * Activate errors for one subsystem.
 *
 * Each executable needs only the error lists of those subsystems it is
 * actually linked with. We always reserve space for NUM_SS char pointers,
 * but only init those of the needed subsystems. This macro is used by macros
 * in config.h (generated by configure).
 */
#define PARA_ERRLIST(ss) [SS_ ## ss] = ss ## _ERRLIST

/**
 * This is temporarily defined to expand to its first argument (prefixed by
 * 'E_') and gets later redefined to expand to the error text only
 */
#define PARA_ERROR(err, msg) E_ ## err
#include "error2.h"
#undef PARA_ERROR
/* Rest of the world only sees the error text. */
#define PARA_ERROR(err, msg) msg
